#include <hc11e1.h>
#include <stdio.h>
#include "drive.h"
#include "utils.h"
#include "exp.h"

WORD lMotorT, rMotorT;
float lOdometer, rOdometer;
int lSpeed, rSpeed, lTicks, rTicks;
unsigned short lTimeOld, rTimeOld;
int lMotorDir,rMotorDir,lMotorSpd,rMotorSpd;
int lPower, rPower;
int lErrorSum, rErrorSum, lErrorPrev, rErrorPrev;
int lError, rError;

// Initialize variables
void driveInit(void)
{
 lMotorDir=FWD;
 rMotorDir=FWD;

 lMotorSpd=0;
 rMotorSpd=0;

 lMotorT=5000;
 rMotorT=5000;

 // Setup OC1 (PWM period)
 _H11TOC1=0;
 bitSet(&_H11OC1M,5,1);  // OC1M5
 bitSet(&_H11OC1M,6,1);  // OC1M6
 bitSet(&_H11OC1D,5,0);  // OC1D5
 bitSet(&_H11OC1D,6,0);  // OC1D6

 bitSet(&_H11TFLG1,7,1); // OC1F
 bitSet(&_H11TMSK1,7,1); // OC1I

 // Setup OC3 (left motor)
 _H11TOC3=_H11TOC1+lMotorT;
 bitSet(&_H11TCTL1,5,1); // OM3
 bitSet(&_H11TCTL1,4,0); // OL3
 bitSet(&_H11TFLG1,5,0); // OC3F
 bitSet(&_H11TMSK1,5,0); // OC3I

 // Setup OC2 (right motor)
 _H11TOC2=_H11TOC1+rMotorT;
 bitSet(&_H11TCTL1,7,1); // OM2
 bitSet(&_H11TCTL1,6,0); // OL2
 bitSet(&_H11TFLG1,6,0); // OC2F
 bitSet(&_H11TMSK1,6,0); // OC2I

 // Initialize the wheel-gear sensor system
 bitSet(&_H11TCTL2,1,0); // EDG3B
 bitSet(&_H11TCTL2,0,1); // EDG3A
 bitSet(&_H11TMSK1,0,1); // IC3I
 bitSet(&_H11TFLG1,0,1); // IC3F
 lOdometer=0;

 bitSet(&_H11TCTL2,3,0); // EDG2B
 bitSet(&_H11TCTL2,2,1); // EDG2A
 bitSet(&_H11TMSK1,1,1); // IC2I
 bitSet(&_H11TFLG1,1,1); // IC2F
 rOdometer=0;

 lTicks=0;
 rTicks=0;
 lSpeed=0;
 rSpeed=0;
 lPower=0;
 rPower=0;
 lError=0;
 rError=0;
 lErrorSum=0;
 rErrorSum=0;
 lErrorPrev=0;
 rErrorPrev=0;
 lMotorDir=FWD;
 lMotorSpd=0;
 rMotorDir=FWD;
 rMotorSpd=0;
}

// left wheel gear sensor - PA0/IC3
interrupt void tic3Handler(void)
{
 lOdometer+=0.418879;
 if(lOdometer>100000) lOdometer=0;
 lTicks++;
 _H11TFLG1|=0x01;
}

// right wheel gear sensor - PA1/IC2
interrupt void tic2Handler(void)
{
 rOdometer+=0.418879;
 if(rOdometer>100000) rOdometer=0;
 rTicks++;
 _H11TFLG1|=0x02;
}

// Handle OC1 and PWM timing
interrupt void toc1Handler(void)
{
 _H11TOC1=_H11TCNT+10000;
 _H11TOC3=_H11TCNT+lMotorT;
 _H11TOC2=_H11TCNT+rMotorT;
 _H11TFLG1|=0x80;
}

// control motor direction and speed
void driveSet(int motor, int dir, int speed)
{
 int dbit;

 if(motor==LEFT)
 {
  dbit=1;
 }
 if(motor==RIGHT)
 {
  dbit=0;
 }

 if((speed>0)&&(speed<10)) speed=10;

 if(dir==FWD)
  expOut(EXP_OUT0,dbit,ON);
 if(dir==REV)
  expOut(EXP_OUT0,dbit,OFF);

 if(speed==0)
 {
  if(motor==LEFT)
  {
   lMotorT=0;
   bitSet(&_H11OC1D,5,0);  // OC1D5
   bitSet(&_H11TCTL1,5,0); // OM3
   bitSet(&_H11TCTL1,4,0); // OL3
  }
  if(motor==RIGHT)
  {
   rMotorT=0;
   bitSet(&_H11OC1D,6,0);  // OC1D6
   bitSet(&_H11TCTL1,7,0); // OM2
   bitSet(&_H11TCTL1,6,0); // OL2
  }
 }
 else if(speed==100)
 {
  if(motor==LEFT)
  {
   lMotorT=5000;
   bitSet(&_H11OC1D,5,1);  // OC1D5
   bitSet(&_H11TCTL1,5,0); // OM3
   bitSet(&_H11TCTL1,4,0); // OL3
  }
  if(motor==RIGHT)
  {
   rMotorT=5000;
   bitSet(&_H11OC1D,6,1);  // OC1D6
   bitSet(&_H11TCTL1,7,0); // OM2
   bitSet(&_H11TCTL1,6,0); // OL2
  }
 }
 else if(motor==LEFT) // PA5/OC3
 {
  lMotorT=10000-(speed*100);
  bitSet(&_H11OC1D,5,0);  // OC1D5
  bitSet(&_H11TCTL1,5,1); // OM3
  bitSet(&_H11TCTL1,4,1); // OL3
 }
 else if(motor==RIGHT) // PA6/OC2
 {
  rMotorT=10000-(speed*100);
  bitSet(&_H11OC1D,6,0);  // OC1D6
  bitSet(&_H11TCTL1,7,1); // OM2
  bitSet(&_H11TCTL1,6,1); // OL2
 }
}

// external interface to motor direction and speed
void driveControl(int motor, int dir, int speed)
{
 if(motor==LEFT)
 {
  lMotorDir=dir;
  lMotorSpd=speed;
 }
 if(motor==RIGHT)
 {
  rMotorDir=dir;
  rMotorSpd=speed;
 }
}

int driveOdometerL(void)
{
 return (int)lOdometer;
}

int driveOdometerR(void)
{
 return (int)rOdometer;
}

int driveSpd(int motor)
{
 if(motor==LEFT)  return lMotorSpd;
 if(motor==RIGHT) return rMotorSpd;
}

int driveDir(int motor)
{
 if(motor==LEFT)  return lMotorDir;
 if(motor==RIGHT) return rMotorDir;
}

int driveVel(int motor)
{
 if(motor==LEFT)
 {
  return lSpeed;
 }
 if(motor==RIGHT)
 {
  return rSpeed;
 }
}

// closed-loop drive control routine

void driveSpeedControl(void)
{
 int lSpd, rSpd;
 int lErrorDiff, rErrorDiff;

 // capture feedback
 lSpeed=((lSpeed*2)+(lTicks*3))/3;
 rSpeed=((rSpeed*2)+(rTicks*3))/3;
 lTicks=0;
 rTicks=0;

 // translate % speeds into real speeds
 lSpd=intMin(lMotorSpd-20,0);
 rSpd=intMin(rMotorSpd-20,0);

 lErrorPrev=lError;
 rErrorPrev=rError;

 // calculate power adjustment

 lError=(lSpd-lSpeed);
 lErrorSum+=lError;
 lErrorDiff=lError-lErrorPrev;
 lPower=(lError/2)+(lErrorSum/2)+(0*lErrorDiff);

 rError=(rSpd-rSpeed);
 rErrorSum+=rError;
 rErrorDiff=rError-rErrorPrev;
 rPower=(rError/2)+(rErrorSum/2)+(0*rErrorDiff);

 intMinMax(&lPower,0,100);
 intMinMax(&rPower,0,100);

 if(lMotorSpd==0) lPower=0;
 if(rMotorSpd==0) rPower=0;

 // set new power level
 driveSet(LEFT,lMotorDir,lPower);
 driveSet(RIGHT,rMotorDir,rPower);
}
