#include <hc11e1.h>
#include <stdlib.h>
#include "utils.h"
#include "drive.h"

int mCnt,lval=0,rval=0;
unsigned short tofCounter;
int sLTime, sRTime, posL, posR;

void bitSet(volatile unsigned char *b,int n, int s)
{
 if(s==ON)
  *b|=bit(n);
 if(s==OFF)
  if((*b>>n)&1)
   *b^=bit(n);
}

void delay(int t)
{
 int i;
 for(i=0;i<=(20*t);i++);
}

// delay in 1 ms increments
void delayms(int t)
{
 int i,dly;
 for(i=1;i<=t;i++)
 {
  dly=_H11TCNT+2000;
  while(_H11TCNT<dly);
 }
}

void intMinMax(int *num, int min, int max)
{
 if(*num>max) *num=max;
 if(*num<min) *num=min;
}

int intMin(int num, int min)
{
 if(num<min)
  return min;
 else
  return num;
}

int intMax(int num, int max)
{
 if(num>max)
  return max;
 else
  return num;
}

interrupt void tofHandler(void)
{
 _H11CFORC|=0x18;
 _H11TOC4=_H11TCNT+sLTime;
 _H11TI4O5=_H11TCNT+sRTime;
 if(tofCounter==65535)
  tofCounter=0;
 else
  tofCounter++;
 mCnt++;
 if(mCnt>=10)
 {
  driveSpeedControl();
  mCnt=0;
 }
 _H11TFLG2|=0x80;
}

unsigned short getTOF(void)
{
 return tofCounter;
}

void servoPos(int servo, int pos)
{
 if(servo==LEFT)
 {
  sLTime=1550+(pos*33);
  delayms(abs(pos-posL));
  posL=pos;
 }
 if(servo==RIGHT)
 {
  sRTime=1550+(pos*33);
  delayms(abs(pos-posR));
  posR=pos;
 }
}

int getServoPos(int servo)
{
 if(servo==LEFT)  return posL;
 if(servo==RIGHT) return posR;
}

void initGeneral(void)
{
 // initialize A/D system for PORTE,0-3
 bitSet(&_H11ADCTL,5,1); // SCAN
 bitSet(&_H11ADCTL,4,1); // MULT
 bitSet(&_H11ADCTL,3,0); // CD
 bitSet(&_H11ADCTL,2,0); // CC

 // Initialize free-running TOF counter
 tofCounter=0;
 mCnt=0;
 bitSet(&_H11TMSK2,7,1); // TOI
 bitSet(&_H11TFLG2,7,1); // TOF

 // Initialize servo timing
 sLTime=3200;
 _H11TOC4=3200;
 bitSet(&_H11TCTL1,3,0); // OM4
 bitSet(&_H11TCTL1,2,1); // OL4
 bitSet(&_H11TFLG1,4,0); // OC4F
 bitSet(&_H11TMSK1,4,0); // OC4I

 sRTime=3200;
 _H11TI4O5=3200;
 bitSet(&_H11TCTL1,1,0); // OM5
 bitSet(&_H11TCTL1,0,1); // OL5
 bitSet(&_H11TMSK1,3,0); // OC5I
 bitSet(&_H11TFLG1,3,0); // OC5F
 
 posL=50;
 posR=50;
}

void haltGeneral(void)
{
 // disable TOF interrupt
 bitSet(&_H11TMSK2,7,0); // TOI
 bitSet(&_H11TFLG2,7,0); // TOF

 // shutdown servo timing
 bitSet(&_H11TCTL1,3,0); // OM4
 bitSet(&_H11TCTL1,2,0); // OL4

 bitSet(&_H11TCTL1,1,0); // OM5
 bitSet(&_H11TCTL1,0,0); // OL5
}
